// Copyright (c) 2014, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
// All rights reserved.
//
// This file is licensed according to the BSD 3-clause license as follows:
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
//       the names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// This file is part of X2C. http://x2c.lcm.at/
// $LastChangedRevision: 2938 $
// $LastChangedDate:: 2023-09-18 16:14:50 +0200#$

function [x,y,typ] = x2c_ModelTransformation(job, arg1, arg2)
    // import necessary java classes
    jimport at.lcm.x2c.utils.Utils;
    
    select job
    case "plot" then
        standard_draw(arg1)
        x = [];
        y = [];
        typ = [];

    case "getinputs" then
        [x, y, typ] = standard_inputs(arg1)

    case "getoutputs" then
        [x, y, typ] = standard_outputs(arg1)

    case "getorigin" then
        [x, y] = standard_origin(arg1)
        typ = [];

    case "set" then
        while %t do
            if ~ exists("%scicos_prob") then
                x2c_localvar = struct();
                x2c_localvar.w = createWindow();
                x2c_localvar.w.axes_size = [500,150];
                x2c_localvar.w.figure_name = "Model Transformation (%d)";
                x2c_localvar.w.icon = fullfile(jinvoke(jinvoke(Utils, "getRootDirectory"), "toString"), "Documentation", "General", "Styles", "X2C", "AppIcon.png");
                x2c_localvar.tf = uicontrol(x2c_localvar.w, "style", "text", "units", "normalized", "position", [0 0.5 1 0.5], "horizontalalignment", "center", "verticalalignment", "bottom");
                x2c_localvar.tf.string = "Model transformation in progress...";
                disp(x2c_localvar.tf.string);    // display string in console
                x2c_localvar.tf2 = uicontrol(x2c_localvar.w, "style", "text", "units", "normalized", "position", [0 0 1 0.5], "horizontalalignment", "center","verticalalignment", "top");
                x2c_localvar.tf2.string = "(If transformation is successful, this window will be automatically closed.)";

                try
                    x2c_transformModel_Comm();
                    x2c_localvar.tf.string = "Model transformation done!";
                    x2c_localvar.tf2.string = "";
                    disp(x2c_localvar.tf.string);        // display string in console
                    sleep(500);             // wait a little before closing the window
                    close(x2c_localvar.w);  // close window
                catch
                    errorStr = lasterror();
                    x2c_localvar.tf.string = "Model transformation failed!";
                    disp(x2c_localvar.tf.string);            // display string in console
                    x2c_localvar.tf2.string = errorStr(2);   // second line of error string should contain the most useful information
                    tf3 = uicontrol(x2c_localvar.w, "style", "text", "units", "normalized", "position", [0 0 1 0.3], "horizontalalignment", "center","verticalalignment", "middle");
                    tf3.string = "(See console output for more information.)";
                    disp(errorStr);     // display full error message in console
                    // do not close window - user has to close it manually
                end

            end
            break
        end
        
        x = arg1;
        y = [];
        typ = [];

    case "define" then
        // see: help scicos_model
        model = scicos_model()
        model.sim = list("x2c_DoNothing_C", 4);
        model.in = [];
        model.evtin = []
        model.out = [];
        model.state = []   // continuous states
        model.dstate = []  // discrete-time states
        model.ipar = [] //contrParamVals
        model.blocktype = "c"
        model.nmode = 0
        model.nzcross = 0
        model.dep_ut = [%f %f]
        exprs = [""]
        gr_i=[];
        x = standard_define([7 3], model, exprs, gr_i);
        x.graphics.style = "fillColor=#ff8c00;gradientColor=#ff8c00;rounded=false;noLabel=false;displayedLabel=transform Model and<BR>push to Communicator;align=center";
        y = [];
        typ = [];
    end

endfunction
